program FindRepeat;
uses Math;

const
  TESTDATA = 'Test a random text. Repeat: a random text';


function EqualSubText(AText: Ansistring; AStart1, AStart2, AMaxLen: Integer): AnsiString;
var
  EqualLen: Integer;
begin
  EqualLen := 0;
  while (AMaxLen > EqualLen) and (AText[AStart1 + EqualLen] = AText[AStart2 + EqualLen]) do
    inc(EqualLen);

  Result := copy(AText, AStart1, EqualLen);
end;

(* DoFindLongestRepeat
   AStart1: Iterates over all potential start positions for the first match of the text
   AMaxSearchLen1: Count of chars up to the start of for the second match
   AStart2: Iterates over all potential start positions for the second match of the text
            Must always be greater the AStart1
            (or equal, which will be handled by "AMaxSearchLen1 = 0")
   AMaxSearchLen2: Count of chars up to the end of the string
*)
function DoFindLongestRepeat(AText, AFound: Ansistring;
    AStart1, AMaxSearchLen1,
    AStart2, AMaxSearchLen2: Integer
  ): Ansistring;
var
  EqualTxt: String;
begin
  Result := AFound;

  EqualTxt := EqualSubText(AText, AStart1, AStart2, Min(AMaxSearchLen1, AMaxSearchLen2));
  if Length(EqualTxt) > Length(Result) then
    Result := EqualTxt;

  if AMaxSearchLen2 > 1 then
    Result := DoFindLongestRepeat(AText, Result,
      // AStart2 increases, so there is one more char available after AStart1
      AStart1,     AMaxSearchLen1 + 1,
      // And there is one char less after AStart2
      AStart2 + 1, AMaxSearchLen2 - 1
    )
  else
  if AStart1 < Length(AText) - 1 then begin
    Result := DoFindLongestRepeat(AText, Result,
      // AStart2 is set equal to AStart1, so AMaxSearchLen1 will be 0
      AStart1 + 1, 0,
      // AStart2 will be 1 more than AStart1 was,
      // so there will be 1 char less to search after AStart2
      AStart1 + 1, AMaxSearchLen1 - 1
    );
  end;
end;

function FindLongestRepeat(AText: Ansistring): Ansistring;
begin
  Result := DoFindLongestRepeat(AText, '',
    1, 0,             // AStart1 equals AStart2: There are 0 chars between
    1, Length(AText)  // AStart2 has the entire string
  );
end;

begin
  writeLn('"' + FindLongestRepeat(TESTDATA) + '"');
  readln;
end.

